/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : Don Sep  6 12:02:57 CEST 2001
    copyright            : (C) 2001 by Michael Speck
    email                : kulkanie@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include "lbreakout.h"
#include "file.h"
#include "chart.h"
#include "config.h"
#include "event.h"
#include "levels.h"
#include "game.h"
#ifdef SOUND
#include "audio.h"
#endif
#include "manager.h"
#include "editor.h"

int term_game = 0;
extern Sdl sdl;
extern Config config;
extern char **levelset_home_names;
extern Font *font;

int main(int argc, char *argv[])
{
    int result = ACTION_NONE;
    int leave = 0;
    char *editor_file = 0;
    char path[512];

#ifdef __BEOS__
int counti;
char datapath[256];
for (counti=strlen(argv[0]) ; argv[0][counti]!='/' ; counti--);
strncpy(datapath, argv[0], counti);
chdir(datapath);
#endif


    /* lbreakout info */
    printf( "LBreakout2 %s\nCopyright 2001 Michael Speck\nPublished under GNU GPL\n---\n", VERSION );
    printf( "Looking up data in: %s\n", SRC_DIR );
#ifndef SOUND
    printf( "Compiled without sound and music\n" );
#endif

    set_random_seed(); /* set random seed */

    config_load();

    init_sdl( SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER );
    set_video_mode( std_video_mode( config.fullscreen ) );
    SDL_WM_SetCaption( "LBreakout2", 0 );
    sdl.fade = config.fade;
    SDL_SetEventFilter( event_filter );
#ifdef SOUND
    audio_open();
    sound_enable( config.sound );
    sound_volume( config.volume * 16 );
#endif

    /* create */
    game_create();
    editor_create();
    manager_create();
    chart_load();
    /* run game */
    manager_fade( FADE_IN );
    while( !leave && !term_game ) {
        result = manager_run();
        switch( result ) {
            case ACTION_QUIT: leave = 1; break;
            case ACTION_PLAY:
                manager_fade( FADE_OUT );
                if ( game_init() ) game_run();
                game_clear();
                manager_fade( FADE_IN );
                break;
            case ACTION_EDIT:
                /* new set? */
                if ( strequal( "<CREATE SET>", levelset_home_names[config.levelset_home_id] ) ) {
                    editor_file = calloc( 16, sizeof( char ) );
                    sprintf( path, "%s/%s/levels", getenv( "HOME" ), CONFIG_DIR_NAME );
                    if ( !enter_string( font, "Set Name:", editor_file, 12 ) || !file_check( path, editor_file, "w" ) ) {
                        free( editor_file );
                        break;
                    }
                    else
                        manager_update_set_list();
                }
                else
                    editor_file = strdup( levelset_home_names[config.levelset_home_id] );
                if ( editor_init( editor_file ) ) {
                    manager_fade( FADE_OUT );
                    editor_run();
                    editor_clear();
                    manager_fade( FADE_IN );
                }
                free( editor_file ); editor_file = 0;
                break;
            default: break;
        }
    }
    manager_fade( FADE_OUT );
    /* delete stuff */
    manager_delete();
    chart_delete();
    editor_delete();
    game_delete();

#ifdef SOUND
    audio_close();
#endif
    chart_save(); /* is okay as chart itself is static and not deleted by chart_delete() */
    config_save();

    return EXIT_SUCCESS;
}
