/***************************************************************************
                          bricks.c  -  description
                             -------------------
    begin                : Thu Sep 6 2001
    copyright            : (C) 2001 by Michael Speck
    email                : kulkanie@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lbreakout.h"
#include "config.h"
#include "levels.h"
#include "player.h"
#include "shrapnells.h"
#include "bricks.h"
#ifdef SOUND
#include "audio.h"
#endif

extern Sdl sdl;
extern SDL_Surface *offscreen; /* offscreen with bricks, background, frame */
extern SDL_Surface *bkgnd; /* background picture */
Brick bricks[MAP_WIDTH][MAP_HEIGHT];
int brick_count; /* how many bricks out there? used to determine when level cleared */
SDL_Surface *brick_pic = 0; /* brick graphics in a horizontal order */
extern Player *player; /* ad new brick score to this player */
enum { EXTRA_STACK_LIMIT = 20 };
typedef struct { int type; int px, py; } Stack_Extra;
Stack_Extra extra_stack[EXTRA_STACK_LIMIT]; /* stack'ed extras released in brick_hit */
int extra_count;
extern int shadow_size;
SDL_Surface *brick_shadow; /* shadow mask */
extern Config config;
extern int shine_x, shine_y, shine_recreate;
extern int active[EX_NUMBER];
#ifdef SOUND
Sound_Chunk *wav_boom = 0;
#endif

/*
====================================================================
Locals
====================================================================
*/

/*
====================================================================
Remove brick from offscreen and screen.
====================================================================
*/
void brick_remove( int mx, int my )
{
    int px, py, w, h, i, j, shadow;
    bricks[mx][my].id = -1;
    bricks[mx][my].type = MAP_EMPTY;
    px = mx * BRICK_WIDTH;
    py = my * BRICK_HEIGHT;
    /* clear offscreen */
    w = BRICK_WIDTH; h = BRICK_HEIGHT;
    if ( config.shadow ) {
        w += shadow_size; h += shadow_size;
        if ( px + w > sdl.screen->w - BRICK_WIDTH ) w = sdl.screen->w - BRICK_WIDTH - px;
        set_surf_clip( offscreen, px, py, w, h );
    }
    DEST(offscreen, px, py, w, h );
    SOURCE(bkgnd, px, py);
    blit_surf();
    /* if shadow redraw close bricks */
    if ( config.shadow ) {
        for ( i = mx - 1; i <= mx + 1; i++ )
            for ( j = my - 1; j <= my + 1; j++ ) {
                if ( i > 0 && j > 0 && i < MAP_WIDTH - 1 ) {
                    if ( bricks[i][j].type != MAP_EMPTY ) {
                        if ( i <= mx && j <= my ) shadow = 1; else shadow = 0;
                        brick_draw( offscreen, i, j, shadow );
                    }
                }
                /* redraw shadow of frame */
                if ( ( i == 0 || j == 0 ) && i < MAP_WIDTH - 1 )
                    brick_draw( offscreen, i, j, 1 );
            }
        set_surf_clip( offscreen, 0, 0, 0, 0 );
    }
    /* update screen */
    DEST(sdl.screen, px, py, w, h );
    SOURCE(offscreen, px, py);
    blit_surf();
    add_refresh_rect(px, py, w, h );
#ifdef SOUND
    sound_play( wav_boom );
#endif
}

/*
====================================================================
Publics
====================================================================
*/

/*
====================================================================
Load picture containing brick graphics
====================================================================
*/
void bricks_load()
{
    brick_pic = load_surf( "bricks.bmp", SDL_HWSURFACE );
    brick_shadow = create_shadow( brick_pic, 0, 0, BRICK_WIDTH, BRICK_HEIGHT );
#ifdef SOUND
    wav_boom = sound_chunk_load( "exp.wav" );
#endif
}
/*
====================================================================
Delete bricks picture
====================================================================
*/
void bricks_delete()
{
    if ( brick_pic ) SDL_FreeSurface( brick_pic ); brick_pic = 0;
    if ( brick_shadow ) SDL_FreeSurface ( brick_shadow ); brick_shadow = 0;
#ifdef SOUND
    if ( wav_boom ) sound_chunk_free( wav_boom ); wav_boom = 0;
#endif
}
/*
====================================================================
Create bricks from level info.
====================================================================
*/
void bricks_create( Level *level )
{
    int i, j;

    /* build walls */
    for (i = 0; i < MAP_WIDTH; i++) {
        bricks[i][0].id = 0;
        bricks[i][0].type = MAP_WALL;
        bricks[i][MAP_HEIGHT - 1].id = -1;
        bricks[i][MAP_HEIGHT - 1].type = MAP_EMPTY;
    }
    for (j = 0; j < MAP_HEIGHT; j++) {
        bricks[0][j].id = 0;
        bricks[0][j].type = MAP_WALL;
        bricks[MAP_WIDTH - 1][j].id = 0;
        bricks[MAP_WIDTH - 1][j].type = MAP_WALL;
    }
    /* clear last lines as there are non-editable */
    for (i = 1; i < MAP_WIDTH - 1; i++)
        for (j = EDIT_HEIGHT + 1; j < MAP_HEIGHT - 1; j++) {
            bricks[i][j].id = -1;
            bricks[i][j].type = MAP_EMPTY;
        }

    /* load map */
    for (i = 0; i < EDIT_WIDTH; i++)
        for (j = 0; j < EDIT_HEIGHT; j++) {
            /* create map */
            switch( level->bricks[i][j] ) {
                case '#':
                    /* wall */
                    bricks[i + 1][j + 1].type = MAP_WALL;
                    bricks[i + 1][j + 1].id = 0;
                    bricks[i + 1][j + 1].score = 0;
                    bricks[i + 1][j + 1].dur = 1;
                    break;
                case 'a':
                case 'b':
                case 'c':
                case 'd':
                case 'e':
                case 'f':
                case 'g':
                case 'h':
                case 'i':
                    /* brick */
                    bricks[i + 1][j + 1].type = MAP_BRICK;
                    bricks[i + 1][j + 1].id = level->bricks[i][j] - 96;
                    bricks[i + 1][j + 1].dur = 1;
                    /* some bricks need more shots */
                    if (bricks[i + 1][j + 1].id < 4)
                        bricks[i + 1][j + 1].dur = bricks[i + 1][j + 1].id;
                    bricks[i + 1][j + 1].score = BRICK_SCORE * bricks[i + 1][j + 1].dur;
                    break;
                default:
                    /* empty */
                    bricks[i + 1][j + 1].type = MAP_EMPTY;
                    bricks[i + 1][j + 1].id = -1;
                    break;
            }
            /* create extras */
            switch ( level->extras[i][j] ) {
                case '0':
                    bricks[i + 1][j + 1].extra = EX_SCORE200;
                    break;
                case '1':
                    bricks[i + 1][j + 1].extra = EX_SCORE500;
                    break;
                case '2':
                    bricks[i + 1][j + 1].extra = EX_SCORE1000;
                    break;
                case '3':
                    bricks[i + 1][j + 1].extra = EX_SCORE2000;
                    break;
                case '4':
                    bricks[i + 1][j + 1].extra = EX_SCORE5000;
                    break;
                case '5':
                    bricks[i + 1][j + 1].extra = EX_SCORE10000;
                    break;
                case 'g':
                    bricks[i + 1][j + 1].extra = EX_GOLDSHOWER;
                    break;
                case '+':
                    bricks[i + 1][j + 1].extra = EX_LENGTHEN;
                    break;
                case '-':
                    bricks[i + 1][j + 1].extra = EX_SHORTEN;
                    break;
                case 'l':
                    bricks[i + 1][j + 1].extra = EX_LIFE;
                    break;
                case 's':
                    bricks[i + 1][j + 1].extra = EX_SLIME;
                    break;
                case 'm':
                    bricks[i + 1][j + 1].extra = EX_METAL;
                    break;
                case 'b':
                    bricks[i + 1][j + 1].extra = EX_BALL;
                    break;
                case 'w':
                    bricks[i + 1][j + 1].extra = EX_WALL;
                    break;
                case 'f':
                    bricks[i + 1][j + 1].extra = EX_FROZEN;
                    break;
                case 'p':
                    bricks[i + 1][j + 1].extra = EX_WEAPON;
                    break;
                case '?':
                    bricks[i + 1][j + 1].extra = EX_RANDOM;
                    break;
                case '>':
                    bricks[i + 1][j + 1].extra = EX_FAST;
                    break;
                case '<':
                    bricks[i + 1][j + 1].extra = EX_SLOW;
                    break;
                case 'j':
                    bricks[i + 1][j + 1].extra = EX_JOKER;
                    break;
                default:
                    bricks[i + 1][j + 1].extra = EX_NONE;
                    break;
            }
        }

    /* count bricks */
    brick_count = 0;
    for (i = 1; i < MAP_WIDTH - 1; i++)
        for (j = 1; j < MAP_HEIGHT - 1; j++)
            if ( bricks[i][j].id > 0 )
                brick_count++;
}
/*
====================================================================
Draw all bricks to offscreen surface.
====================================================================
*/
void bricks_draw()
{
    int i, j;
    if ( offscreen == 0 ) return;
    set_surf_clip( offscreen, 0, 0, sdl.screen->w - BRICK_WIDTH, sdl.screen->h );
    for ( j = 1; j < MAP_HEIGHT ; j++ )
        for ( i = 1; i < MAP_WIDTH - 1; i++ )
            if ( bricks[i][j].id >= 0 )
                brick_draw( offscreen, i, j, config.shadow );
    set_surf_clip( offscreen, 0,0,0,0 );
}
/*
====================================================================
Draw brick to passed surface
====================================================================
*/
void brick_draw( SDL_Surface *surf, int map_x, int map_y, int shadow )
{
    int x = map_x * BRICK_WIDTH, y = map_y * BRICK_HEIGHT;
    /* add shadow */
    if ( shadow ) {
        DEST( surf, x + shadow_size, y + shadow_size, BRICK_WIDTH, BRICK_HEIGHT );
        SOURCE( brick_shadow, 0, 0 );
        alpha_blit_surf( SHADOW_ALPHA );
    }
    /* brick if not frame brick */
    if ( map_x == 0 || map_y == 0 || map_x == MAP_WIDTH - 1 ) return;
    DEST( surf, x, y, BRICK_WIDTH, BRICK_HEIGHT );
    SOURCE( brick_pic, bricks[map_x][map_y].id * BRICK_WIDTH, 0 );
    blit_surf();
}
/*
====================================================================
Hit brick and remove if destroyed. 'metal' means the ball
destroys any brick with the first try.
Return true on destruction
====================================================================
*/
int brick_hit( int mx, int my, int metal, int res )
{
    int px, py;
    int was_wall = 0;
    int remove = 0;
    int loose_dur = 0;

    /* outer walls cannot be removed */
    if ( mx <= 0 || mx >= MAP_WIDTH - 1 || my <= 0 || my >= MAP_HEIGHT - 1 ) return 0;

    /* if metal ball resistance is futile */
    if ( metal )
        remove = 1;
    else {
        if ( bricks[mx][my].id <= 0 ) return 0; /* walls can't be broken by normal ball */
        if ( bricks[mx][my].dur <= 1 )
            remove = 1;
        else
            loose_dur = 1;
    }

    if ( remove ) {
        /* mark as removed in players brick map */
        player->bricks[mx][my] = 0; /* duration of 0 means destruction */
        /* get screen position */
        px = mx * BRICK_WIDTH;
        py = my * BRICK_HEIGHT;
        /* add shrapnells */
        shrapnells_create( px, py, BRICK_WIDTH, BRICK_HEIGHT, res );
        /* remove */
        if ( bricks[mx][my].id == 0 ) was_wall = 1;
        brick_remove( mx, my );
        /* release extra if one exists */
        if ( bricks[mx][my].extra != EX_NONE )
            bricks_push_extra( bricks[mx][my].extra, px, py );
        else
            if ( active[EX_GOLDSHOWER] )
                bricks_push_extra( EX_SCORE1000, px, py );
        /* get score */
        player->score += bricks[mx][my].score;
        /* decrease brick counter */
        if ( !was_wall ) --brick_count;
        /* recreate shine if needed */
        if (px == shine_x && py == shine_y) shine_recreate = 1;
    }
    else
        if ( loose_dur ) {
            bricks[mx][my].dur--;
            bricks[mx][my].id--;
            player->bricks[mx][my] = bricks[mx][my].dur;
            brick_draw( offscreen, mx, my, 0 );
            brick_draw( sdl.screen, mx, my, 0 );
            px = mx * BRICK_WIDTH;
            py = my * BRICK_HEIGHT;
            add_refresh_rect(px, py, BRICK_WIDTH, BRICK_HEIGHT);
        }
    return remove;
}
/*
====================================================================
Get released extra from queue if any.
====================================================================
*/
void bricks_push_extra( int type, int px, int py )
{
    if ( extra_count == EXTRA_STACK_LIMIT ) return;
    extra_stack[extra_count].type = type;
    extra_stack[extra_count].px = px;
    extra_stack[extra_count].py = py;
    extra_count++;
}
int bricks_pop_extra( int *type, int *px, int *py )
{
    if ( extra_count == 0 ) return 0;
    extra_count--;
    *type = extra_stack[extra_count].type;
    *px = extra_stack[extra_count].px;
    *py = extra_stack[extra_count].py;
    return 1;
}

