/***************************************************************************
                          extras.c  -  description
                             -------------------
    begin                : Sun Sep 9 2001
    copyright            : (C) 2001 by Michael Speck
    email                : kulkanie@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lbreakout.h"
#include "list.h"
#include "config.h"
#include "event.h"
#include "difficulty.h"
#include "levels.h"
#include "player.h"
#include "bricks.h"
#include "shots.h"
#include "paddle.h"
#include "balls.h"
#include "extras.h"
#include "frame.h"
#ifdef SOUND
#include "audio.h"
#endif

extern Diff *diff;
extern Player *player;
extern Config config;
extern Sdl sdl;
extern SDL_Surface *offscreen;
extern Paddle paddle;
extern int ball_w, ball_dia;
extern float ball_v, ball_vm, ball_old_v;
extern List *balls;
extern SDL_Surface *brick_pic;
extern Brick bricks[MAP_WIDTH][MAP_HEIGHT];
SDL_Surface *extra_pic = 0; /* graphics */
SDL_Surface *extra_shadow = 0;
List *extras = 0; /* extras */
int active[EX_NUMBER]; /* true if extra activated */
int extra_time[EX_NUMBER]; /* time in milliseconds when extra expires: 0 unlimited */
/* wall */
SDL_Surface *wall_pic = 0;
int wall_alpha = 0;
extern int shadow_size;
#ifdef SOUND
Sound_Chunk *wav_score = 0, *wav_metal = 0, *wav_std = 0, *wav_wall = 0, *wav_joker = 0;
Sound_Chunk *wav_goldshower = 0;
Sound_Chunk *wav_speedup = 0, *wav_speeddown = 0;
extern Sound_Chunk *wav_attach; /* handled by balls.c */
#endif

/* extra conversion table */
/* is not used when actually loading a level for playing so check back with
brick.c when adding new extras */
Extra_Conv extra_conv_table[EX_NUMBER] = {
    { EX_SCORE200,   '0' },
    { EX_SCORE500,   '1' },
    { EX_SCORE1000,  '2' },
    { EX_SCORE2000,  '3' },
    { EX_SCORE5000,  '4' },
    { EX_SCORE10000, '5' },
    { EX_GOLDSHOWER, 'g' },
    { EX_LENGTHEN,   '+' },
    { EX_SHORTEN,    '-' },
    { EX_LIFE,       'l' },
    { EX_SLIME,      's' },
    { EX_METAL,      'm' },
    { EX_BALL,       'b' },
    { EX_WALL,       'w' },
    { EX_FROZEN,     'f' },
    { EX_WEAPON,     'p' },
    { EX_RANDOM,     '?' },
    { EX_FAST,       '>' },
    { EX_SLOW,       '<' },
    { EX_JOKER,      'j' }
};

/*
====================================================================
Extra times
====================================================================
*/
enum {
    TIME_WALL   = 15000,
    TIME_METAL  = 8000,
    TIME_WEAPON = 8000,
    TIME_FROZEN = 1000,
    TIME_SLIME  = 20000,
    TIME_FAST   = 20000,
    TIME_SLOW   = 20000,
    TIME_GOLDSHOWER = 20000
};

/*
====================================================================
Locals
====================================================================
*/

/*
====================================================================
Public
====================================================================
*/

/*
====================================================================
Load and delete extra graphics
====================================================================
*/
void extras_load()
{
    int i;

    extra_pic = load_surf( "extras.bmp", SDL_HWSURFACE );
    extras = list_create( LIST_AUTO_DELETE, NO_CALLBACK );
    /* wall */
    wall_pic = create_surf( sdl.screen->w - 2 * BRICK_WIDTH, BRICK_HEIGHT, SDL_HWSURFACE );
    /* bricks must have been loaded at this point! */
    for ( i = 0; i < MAP_WIDTH - 2; i++ ) {
        DEST( wall_pic, i * BRICK_WIDTH, 0, BRICK_WIDTH, BRICK_HEIGHT );
        SOURCE( brick_pic, 0, 0 );
        blit_surf();
    }
    /* shadow */
    extra_shadow = create_shadow( extra_pic, 0, 0, BRICK_WIDTH, BRICK_HEIGHT );
#ifdef SOUND
    wav_score = sound_chunk_load( "score.wav" );
    wav_metal = sound_chunk_load( "metal.wav" );
    wav_speedup = sound_chunk_load( "speedup.wav" );
    wav_speeddown = sound_chunk_load( "speeddown.wav" );
    wav_std = sound_chunk_load( "standard.wav" );
    wav_wall = sound_chunk_load( "wall.wav" );
    wav_joker = sound_chunk_load( "joker.wav" );
    wav_goldshower = sound_chunk_load( "goldshower.wav" );
#endif
}
void extras_delete()
{
    if ( extra_pic ) SDL_FreeSurface( extra_pic ); extra_pic = 0;
    if ( extras ) list_delete( extras ); extras = 0;
    if ( wall_pic ) SDL_FreeSurface( wall_pic ); wall_pic = 0;
    if ( extra_shadow ) SDL_FreeSurface( extra_shadow ); extra_shadow = 0;
#ifdef SOUND
    if ( wav_score ) sound_chunk_free( wav_score ); wav_score = 0;
    if ( wav_metal ) sound_chunk_free( wav_metal ); wav_metal = 0;
    if ( wav_speedup ) sound_chunk_free( wav_speedup ); wav_speedup = 0;
    if ( wav_speeddown ) sound_chunk_free( wav_speeddown ); wav_speeddown = 0;
    if ( wav_std ) sound_chunk_free( wav_std ); wav_std = 0;
    if ( wav_wall ) sound_chunk_free( wav_wall ); wav_wall = 0;
    if ( wav_joker ) sound_chunk_free( wav_joker ); wav_joker = 0;
    if ( wav_goldshower ) sound_chunk_free( wav_goldshower ); wav_goldshower = 0;
#endif
}
/*
====================================================================
Reset extras
====================================================================
*/
void extras_reset()
{
    list_clear( extras );
    memset( active, 0, sizeof( int ) * EX_NUMBER );
    memset( extra_time, 0, sizeof( int ) * EX_NUMBER );
}
/*
====================================================================
Create new extra at position
====================================================================
*/
void extra_create( int type, int x, int y )
{
    Extra *e = calloc( 1, sizeof( Extra ) );
    e->type = type;
    e->x = x; e->y = y;
    if (!config.trp)
        e->alpha = 0;
    else
        e->alpha = 255;
    list_add( extras, e );
}
/*
====================================================================
Use extra when paddle collected it
====================================================================
*/
void extra_use( int extra_type )
{
    Ball    *b;
    int i;

    while( extra_type == EX_RANDOM ) extra_type = rand() % (EX_NUMBER);

    switch (extra_type) {
        case EX_SCORE200:
            player->score += 200;
            if ( active[EX_JOKER] ) player->score += 200;
#ifdef SOUND
            sound_play( wav_score );
#endif
            break;
        case EX_SCORE500:
            player->score += 500;
            if ( active[EX_JOKER] ) player->score += 500;
#ifdef SOUND
            sound_play( wav_score );
#endif
            break;
        case EX_SCORE1000:
            player->score += 1000;
            if ( active[EX_JOKER] ) player->score += 1000;
#ifdef SOUND
            sound_play( wav_score );
#endif
            break;
        case EX_SCORE2000:
            player->score += 2000;
            if ( active[EX_JOKER] ) player->score += 2000;
#ifdef SOUND
            sound_play( wav_score );
#endif
            break;
        case EX_SCORE5000:
            player->score += 5000;
            if ( active[EX_JOKER] ) player->score += 5000;
#ifdef SOUND
            sound_play( wav_score );
#endif
            break;
        case EX_SCORE10000:
            player->score += 10000;
            if ( active[EX_JOKER] ) player->score += 10000;
#ifdef SOUND
            sound_play( wav_score );
#endif
            break;
        case EX_GOLDSHOWER:
            extra_time[EX_GOLDSHOWER] += TIME_GOLDSHOWER;
            if ( active[EX_JOKER] ) extra_time[EX_GOLDSHOWER] += TIME_GOLDSHOWER;
#ifdef SOUND
            sound_play( wav_goldshower );
#endif
            break;
        case EX_LIFE:
            if (player->lives < diff->max_lives) {
                player->lives++;
                /* update lives */
                frame_add_life();
            }
            else {
                player->score += 10000;
#ifdef SOUND
            sound_play( wav_score );
#endif
            }
            break;
        case EX_SHORTEN:
            paddle_init_resize(-1);
            break;
        case EX_LENGTHEN:
            paddle_init_resize(1);
            break;
        case EX_BALL:
            b = ball_create(paddle.x + (paddle.w - ball_w) / 2, paddle.y - ball_dia, (rand() % 90) - 45, 0.15);
            ball_mask_vel(b, b->vel.x);
            ball_get_target(b);
            list_add( balls, b );
#ifdef SOUND
            sound_play( wav_std );
#endif
            break;
        case EX_WALL:
            extra_time[EX_WALL] += TIME_WALL;
            if ( active[EX_JOKER] ) extra_time[EX_WALL] += TIME_WALL;
            if (active[EX_WALL]) {
#ifdef SOUND
                sound_play( wav_std );
#endif
                break;
            }
            for (i = 1; i < MAP_WIDTH - 1; i++) {
                bricks[i][MAP_HEIGHT - 1].type = MAP_WALL;
                bricks[i][MAP_HEIGHT - 1].id = 0;
            }
            wall_alpha = 255;
            balls_get_new_targets( -1, 0 );
#ifdef SOUND
            sound_play( wav_wall );
#endif
            break;
        case EX_METAL:
            extra_time[EX_METAL] += TIME_METAL;
            if ( active[EX_JOKER] ) extra_time[EX_METAL] += TIME_METAL;
            balls_set_metal( 1 );
#ifdef SOUND
            sound_play( wav_metal );
#endif
            break;
        case EX_FROZEN:
            extra_time[EX_FROZEN] = TIME_FROZEN;
            paddle_freeze( 1 );
#ifdef SOUND
#endif
            break;
        case EX_WEAPON:
            extra_time[EX_WEAPON] += TIME_WEAPON;
            if ( active[EX_JOKER] ) extra_time[EX_WEAPON] += TIME_WEAPON;
            weapon_install( 1 );
#ifdef SOUND
            sound_play( wav_std );
#endif
            break;
        case EX_SLIME:
            extra_time[EX_SLIME] += TIME_SLIME;
            if ( active[EX_JOKER] ) extra_time[EX_SLIME] += TIME_SLIME;
            paddle_set_slime( 1 );
#ifdef SOUND
            sound_play( wav_attach );
#endif
            break;
        case EX_FAST:
#ifdef SOUND
            sound_play( wav_speedup );
#endif
            if ( active[EX_SLOW] || active[EX_FAST] )
                ball_v = ball_old_v;
            if ( active[EX_SLOW] ) {
                extra_time[EX_SLOW] = 0;
                active[EX_SLOW] = 0;
            }
            extra_time[EX_FAST] += TIME_FAST;
            if ( active[EX_JOKER] ) extra_time[EX_FAST] += TIME_FAST;
            ball_old_v = ball_v;
            ball_v = ball_vm;
            list_reset( balls );
            while ( ( b = list_next( balls ) ) ) {
                ball_adjust_vel(b, ball_v);
                ball_get_target(b);
            }
            break;
        case EX_SLOW:
#ifdef SOUND
            sound_play( wav_speeddown );
#endif
            if ( active[EX_SLOW] || active[EX_FAST] )
                ball_v = ball_old_v;
            if ( active[EX_FAST] ) {
                extra_time[EX_FAST] = 0;
                active[EX_FAST] = 0;
            }
            extra_time[EX_SLOW] += TIME_SLOW;
            if ( active[EX_JOKER] ) extra_time[EX_SLOW] += TIME_SLOW;
            ball_old_v = ball_v;
            ball_v = diff->v_start;
            list_reset( balls );
            while ( ( b = list_next( balls ) ) ) {
                ball_adjust_vel(b, ball_v);
                ball_get_target(b);
            }
            break;
    }
    active[extra_type] = 1;
}
/*
====================================================================
Show, hide extras
====================================================================
*/
void extras_hide()
{
    List_Entry  *entry = extras->head.next;
    Extra       *ex;
    int         x, y;
    int         w = BRICK_WIDTH, h = BRICK_HEIGHT;
    if ( config.shadow ) { w += shadow_size; h += shadow_size; }
    while ( entry != &extras->tail ) {
        ex = entry->item;
        x = (int)ex->x;
        y = (int)ex->y;
        DEST(sdl.screen, x, y, w, h );
        SOURCE(offscreen, x, y);
        blit_surf();
        add_refresh_rect(x, y, w, h );
        entry = entry->next;
    }
}
void extra_show_shadow( Extra *extra )
{
	DEST( sdl.screen, (int)extra->x + shadow_size, (int)extra->y + shadow_size, BRICK_WIDTH, BRICK_HEIGHT );
    SOURCE( extra_shadow, 0, 0 );
    alpha_blit_surf( ( extra->alpha / 255 ) * ( 255 - SHADOW_ALPHA ) + SHADOW_ALPHA );
}
void extras_show()
{
    List_Entry  *entry = extras->head.next;
    Extra       *ex;
    int         x, y;
    while ( entry != &extras->tail ) {
        ex = entry->item;
        x = (int)ex->x;
        y = (int)ex->y;
        if ( config.shadow ) extra_show_shadow( ex );
        DEST(sdl.screen, x, y, BRICK_WIDTH, BRICK_HEIGHT);
        SOURCE(extra_pic, (ex->type - 1) * BRICK_WIDTH, 0);
        alpha_blit_surf((int)ex->alpha);
        if ( config.shadow )
            add_refresh_rect(x, y, BRICK_WIDTH + shadow_size, BRICK_HEIGHT + shadow_size );
        else
            add_refresh_rect(x, y, BRICK_WIDTH, BRICK_HEIGHT);
        entry = entry->next;
    }
}
void extras_alphashow( int alpha )
{
    List_Entry  *entry = extras->head.next;
    Extra       *ex;
    int         x, y;
    while ( entry != &extras->tail ) {
        ex = entry->item;
        x = (int)ex->x;
        y = (int)ex->y;
        DEST(sdl.screen, x, y, BRICK_WIDTH, BRICK_HEIGHT);
        SOURCE(extra_pic, (ex->type - 1) * BRICK_WIDTH, 0);
        alpha_blit_surf( alpha );
        add_refresh_rect(x, y, BRICK_WIDTH, BRICK_HEIGHT);
        entry = entry->next;
    }
}
/*
====================================================================
Update extras
====================================================================
*/
void extras_update( int ms )
{
    List_Entry  *entry;
    Extra       *ex;
    Ball *ball;
    int i;
    int type, px, py;

    /* release new extras */
    while ( bricks_pop_extra( &type, &px, &py ) ) extra_create( type, px, py );
    /* check extra_time of limited extras */
    for ( i = 0; i < EX_NUMBER; i++ )
        /* extra_time of wall is updated in wall_update() */
        if ( extra_time[i] && i != EX_WALL ) {
            extra_time[i] -= ms;
            if ( extra_time[i] <= 0 ) {
                extra_time[i] = 0;
                /* expired */
                switch ( i ) {
                    case EX_SLIME: paddle_set_slime( 0 ); break;
                    case EX_WEAPON: weapon_install( 0 ); break;
                    case EX_FROZEN:
                        paddle_freeze( 0 );
                        break;
                    case EX_METAL: balls_set_metal( 0 ); break;
                    case EX_SLOW:
                    case EX_FAST:
                        ball_v = ball_old_v;
                        list_reset( balls );
                        while ( ( ball = list_next( balls ) ) ) {
                            ball_adjust_vel(ball, ball_v);
                            ball_get_target(ball);
                        }
                        break;
                }
                /* set deactivated */
                active[i] = 0; /* wall is handled in wall_...() */
            }
        }
    /* move extras and check if paddle was hit */
    entry = extras->head.next;
    while ( entry != &extras->tail ) {
        ex = entry->item;
        ex->y += 0.05 * ms;
        if (ex->alpha > 0)
            ex->alpha -= 0.25 * ms;
        if (ex->alpha < 0)
            ex->alpha = 0;
        entry = entry->next;
        if (ex->y >= sdl.screen->h)
            list_delete_entry( extras, entry->prev );
        else {
            // contact with paddle core ? //
            if (ex->x + BRICK_WIDTH > paddle.x && ex->x < paddle.x + paddle.w - 1 && ex->y + BRICK_HEIGHT > paddle.y && ex->y < paddle.y + paddle.h) {
                /* if ex->type is EX_JOKER check all extras and use all bonus and quit*/
                if ( ex->type == EX_JOKER ) {
#ifdef SOUND
                    sound_play( wav_joker );
#endif
                    /* set EX_JOKER as active so times may be doubled in extra_use */
                    active[EX_JOKER] = 1;
                    list_reset( extras );
                    while ( ( ex = list_next( extras ) ) ) {
                        if ( ex->type != EX_JOKER )
                        if ( ex->type != EX_SHORTEN )
                        if ( ex->type != EX_FROZEN )
                        if ( ex->type != EX_FAST )
                        if ( ex->type != EX_RANDOM )
                            extra_use( ex->type );
                        list_delete_item( extras, ex );
                    }
                    active[EX_JOKER] = 0;
                    break;
                }
                extra_use(ex->type);
                list_delete_entry( extras, entry->prev );
            }
        }
    }
}
/* wall */
void wall_hide()
{
    if ( !active[EX_WALL] ) return;
    DEST( sdl.screen, BRICK_WIDTH, sdl.screen->h - BRICK_HEIGHT, wall_pic->w, wall_pic->h );
    SOURCE( offscreen, BRICK_WIDTH, sdl.screen->h - BRICK_HEIGHT );
    blit_surf();
}
void wall_show()
{
    if ( !active[EX_WALL] ) return;
    DEST( sdl.screen, BRICK_WIDTH, sdl.screen->h - BRICK_HEIGHT, wall_pic->w, wall_pic->h );
    SOURCE( wall_pic, 0, 0 );
    if ( config.trp && wall_alpha )
        alpha_blit_surf( wall_alpha );
    else
        blit_surf();
    add_refresh_rect( BRICK_WIDTH, sdl.screen->h - BRICK_HEIGHT, wall_pic->w, wall_pic->h );
}
void wall_alphashow( int alpha )
{
    if ( !active[EX_WALL] ) return;
    DEST( sdl.screen, BRICK_WIDTH, sdl.screen->h - BRICK_HEIGHT, wall_pic->w, wall_pic->h );
    SOURCE( wall_pic, 0, 0 );
    alpha_blit_surf( alpha );
    add_refresh_rect( BRICK_WIDTH, sdl.screen->h - BRICK_HEIGHT, wall_pic->w, wall_pic->h );
}
void wall_update( int ms )
{
    int i;

    if ( !active[EX_WALL] ) return;
    if ( extra_time[EX_WALL] > 0 ) {
	    extra_time[EX_WALL] -= ms;
		if ( extra_time[EX_WALL] < 0 ) extra_time[EX_WALL] = 0;
        /* still appearing? */
        if (wall_alpha > 0) {
			wall_alpha -= 0.25 * ms;
        	if (wall_alpha < 0) wall_alpha = 0;
		}				
    }
    else {
        wall_alpha += 0.25 * ms;
        if ( wall_alpha >= 255 ) {
            wall_alpha = 255;
            /* disappear */
            wall_hide();
            add_refresh_rect( BRICK_WIDTH, sdl.screen->h - BRICK_HEIGHT, wall_pic->w, wall_pic->h );
            active[EX_WALL] = 0;
            for (i = 1; i < MAP_WIDTH - 1; i++) bricks[i][MAP_HEIGHT - 1].type = MAP_EMPTY;
            balls_get_new_targets( -1, 0 );
        }
    }
}
