/***************************************************************************
                          game.c  -  description
                             -------------------
    begin                : Thu Sep 6 2001
    copyright            : (C) 2001 by Michael Speck
    email                : kulkanie@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lbreakout.h"
#include "event.h"
#include "config.h"
#include "difficulty.h"
#include "shrapnells.h"
#include "levels.h"
#include "player.h"
#include "chart.h"
#include "shine.h"
#include "credit.h"
#include "bricks.h"
#include "shots.h"
#include "frame.h"
#include "paddle.h"
#include "balls.h"
#include "extras.h"
#include "game.h"
#ifdef SOUND
#include "audio.h"
#endif

SDL_Surface *bkgnd = 0; /* current background picture */
SDL_Surface *offscreen = 0; /* buffer with frame, background and bricks */
int screenshot = 0; /* index of current screenshot */
Font *font = 0; /* standard font */
Player *player = 0; /* current player */
extern Config config; /* lbreakout config struct */
extern int term_game; /* terminate game */
extern Sdl sdl; /* SDL struct */
extern Brick bricks[MAP_WIDTH][MAP_HEIGHT];
Diff *diff = 0; /* current difficulty information */
extern int keystate[SDLK_LAST];
extern int buttonstate[BUTTON_COUNT];
extern int active[EX_NUMBER];
extern int shot_bricks_hit, shot_mx, shot_my; /* position at which brick (and maybe neighbor) was destroyed */
extern int brick_count;
extern char **levelset_names; /* names of all available levelsets */
extern Player players[MAX_PLAYERS]; /* player infos */
#ifdef SOUND
Sound_Chunk *wav_click = 0;
Sound_Chunk *wav_damn = 0, *wav_dammit = 0, *wav_wontgiveup = 0, *wav_excellent = 0, *wav_verygood = 0;
#endif

/*
====================================================================
Locals
====================================================================
*/

/*
====================================================================
Initate current level from player's level resource pointer.
Recreates offscreen (frame, background, bricks) and resets paddle,
balls, extras. Do not add the bricks already removed:
player->bricks[x][y] == 0.
Return Value: True if successful
====================================================================
*/
enum { WITH_CREDITS = 1 };
int game_init_level( Player *player, int with_credits )
{
    int i, j;

    /* load bricks&extras from level data */
    bricks_create( player->level );
    /* remove cleared bricks */
    for ( i = 1; i < MAP_WIDTH - 1; i++ )
        for ( j = 1; j < MAP_HEIGHT - 1; j++ ) {
            if ( player->bricks[i][j] == -1 ) {
                /* initiate duration -- 0 means this brick was removed */
                if ( bricks[i][j].id == -1 )
                    player->bricks[i][j] = 0;
                else
                    player->bricks[i][j] = bricks[i][j].dur;
            }
            else
                if ( player->bricks[i][j] == 0 ) {
                    if ( bricks[i][j].id > 0 ) brick_count--;
                    bricks[i][j].id = -1;
                    bricks[i][j].type = MAP_EMPTY;
                }
                else {
                    if ( bricks[i][j].dur > 1 )
                        bricks[i][j].id = player->bricks[i][j];
                    bricks[i][j].dur =  player->bricks[i][j];
                }
        }

    /* recreate offscreen */
    if ( offscreen ) SDL_FreeSurface( offscreen );
    offscreen = create_surf( sdl.screen->w, sdl.screen->h, SDL_SWSURFACE );
    SDL_SetColorKey( offscreen, 0, 0 );

    /* add&create background */
    if ( bkgnd ) SDL_FreeSurface( bkgnd );
    bkgnd = bkgnd_draw( -1 );
    /* add frame */
    frame_draw();
    /* add bricks */
    bricks_draw();
    /* draw lives */
//    player->score = 50000;
    frame_draw_lives( player->lives, diff->max_lives );
    /* reset score */
    frame_score_reset( player->score );
    /* draw player's name */
    frame_draw_player_name( player->name );
    /* reset paddle */
    paddle_reset();
//    weapon_install( 1 );
//    paddle_set_slime( 1 );
    /* event reset */
    event_reset();
    /* shots reset */
    shots_reset();
    /* reset balls */
    balls_set_vel( diff->v_start, diff->v_change, diff->v_max );
    balls_reset();
//    balls_set_metal( 1 );
    /* reset extras */
    extras_reset();
    /* reset shrapnells */
    shrapnells_reset();
    /* reset shine */
    shine_reset();
    /* initiate credit */
    if ( with_credits )
        credit_init( player->level->name, player->level->author, levels_get_id( player->level ) );

    /* show offscreen */
    FULL_DEST( sdl.screen ); FULL_SOURCE( offscreen ); blit_surf();
    /* no refresh rect as we want to use dim effect */
    return 1;
}

/*
====================================================================
Display info about player
====================================================================
*/
void player_info()
{
    char info[128];
    /* short info about actual player */
    /* no info if this is the last active player */
    if ( players_count() > 1 ) {
        sprintf( info, "Next Player: %s", player->name );
        confirm( font, info, CONFIRM_ANY_KEY );
    }
    else
        refresh_screen( 0, 0, 0, 0 );
}
/*
====================================================================
Runs Lbreakout until player looses ball or explicitly wishes
to quit, restart. (just request! confirmation not done here)
Assumes that the current offscreen was already displayed on screen
by game_init_level.
====================================================================
*/
enum {
    REQUEST_NONE = 0,
    REQUEST_RESTART,
    REQUEST_QUIT,
    REQUEST_NEXT_LEVEL,
    REQUEST_NEXT_PLAYER,
    REQUEST_PAUSE
};
int breakout_run() {
    SDL_Event event;
    int result = REQUEST_NONE;
    int ms;
    int fps_delay = 0;

    /* delay */
    switch ( config.fps ) {
        case 1: fps_delay = 20; break;
        case 2: fps_delay = 10; break;
        case 3: fps_delay = 5; break;
    }
    /* grab input if wanted */
    event_grab_input();
    /* run main loop */
    event_reset();
    reset_timer();
    while( result == REQUEST_NONE ) {
        if ( config.fps ) SDL_Delay( fps_delay );
        if ( event_poll( &event ) ) {
            switch ( event.type ) {
                case SDL_QUIT: term_game = 1; result = REQUEST_QUIT; break;
                case SDL_KEYDOWN:
                    if ( keystate[config.k_fire] && weapon_installed() && !weapon_firing() ) weapon_start_fire();
                    break;
                case SDL_KEYUP:
                    if ( weapon_firing() && !keystate[config.k_fire] ) weapon_stop_fire();
                    /* various requests */
                    switch ( event.key.keysym.sym ) {
						case SDLK_q:
                        case SDLK_ESCAPE: result = REQUEST_QUIT; break;
                        case SDLK_r:
                            if (  player->lives <= 1 && player->score < diff->life_cost ) break;
                            result = REQUEST_RESTART;
                            player->lives--;
                            break;
                        case SDLK_f:
                            event_ungrab_input();
                            config.fullscreen = !config.fullscreen;
                            set_video_mode( std_video_mode( config.fullscreen ) );
                            /* redraw offscreen to get background */
                            FULL_DEST( sdl.screen ); FULL_SOURCE( offscreen ); blit_surf();
                            refresh_screen( 0, 0, 0, 0 );
                            event_grab_input();
                            break;
                        case SDLK_s:
#ifdef SOUND
                            config.sound = !config.sound;
                            sound_enable( config.sound );
#endif
                            break;
                        case SDLK_a:
                            config.anim++;
                            if ( config.anim >= 3 ) config.anim = 0;
                            break;
                        case SDLK_TAB:
                            take_screenshot( screenshot++ );
                            break;
                        case SDLK_p:
                            result = REQUEST_PAUSE;
                            break;
                        default: break;
                    }
                    break;
                case SDL_MOUSEBUTTONDOWN:
                    if ( buttonstate[LEFT_BUTTON] && weapon_installed() && !weapon_firing() ) weapon_start_fire();
                    break;
                case SDL_MOUSEBUTTONUP:
                    if ( weapon_firing() && !buttonstate[LEFT_BUTTON] ) weapon_stop_fire();
                    break;
            }
        }
        ms = get_time();
        /* hide */
        frame_info_hide();
        extras_hide();
        paddle_hide();
        balls_hide();
        shots_hide();
        shrapnells_hide();
        wall_hide();
        frame_score_hide();
        shine_hide();
        credit_hide();
        /* update */
        paddle_update( ms );
        if ( !balls_update( ms ) ) {
            result = REQUEST_NEXT_PLAYER;
            player->lives--;
        }
        shots_update( ms );
        extras_update( ms );
        wall_update( ms );
        shrapnells_update( ms );
        frame_score_update( ms );
        shine_update( ms );
        credit_update( ms );
        /* check if bricks were destroyed by shots */
        if ( shot_bricks_hit ) {
            balls_get_new_targets( shot_mx, shot_my );
            if ( shot_bricks_hit == 2 ) /* hit neighbor, too */
                balls_get_new_targets( shot_mx + 1, shot_my );
            shot_bricks_hit = 0;
        }
        /* update score */
        if ( player_score_changed( player ) ) frame_score_set( player->score );
        /* update ball speed */
        if ( !active[EX_SLOW] && !active[EX_FAST] ) balls_inc_vel( ms );
        /* check if all bricks where destroyed */
        if ( brick_count == 0 ) result = REQUEST_NEXT_LEVEL;
        /* show */
        if ( config.ball_level == BALL_BELOW_BONUS )
			balls_show();
        shots_show();
        extras_show();
		if ( config.ball_level == BALL_ABOVE_BONUS )
			balls_show();
        paddle_show();
        wall_show();
        shine_show();
        shrapnells_show();
        frame_score_show();
        frame_info_show();
        credit_show();
        /* update anything that was changed */
        refresh_rects();
    }
    /* release input */
    event_ungrab_input();
    return result;
}
/*
====================================================================
Fade all animations until they disappear
====================================================================
*/
void fade_anims()
{
    float alpha = 0;
    int ms;
    frame_remove_life();
    reset_timer();
    while ( alpha < 255 ) {
        paddle_hide();
        balls_hide();
        extras_hide();
        shrapnells_hide();
        shots_hide();
        wall_hide();
        credit_hide();
        ms = get_time();
        alpha += 0.5 * ms;
        if ( alpha > 255 ) alpha = 255;
        shrapnells_update( ms );
        paddle_alphashow( alpha );
        balls_alphashow( alpha );
        extras_alphashow( alpha );
        shots_alphashow( alpha );
        shrapnells_show();
        wall_alphashow( alpha );
        credit_alphashow( alpha );
        refresh_rects();
    }
}

/*
====================================================================
Publics
====================================================================
*/

/*
====================================================================
Load all static resources like frame, bricks, balls, extras...
====================================================================
*/
void game_create()
{
    font = load_fixed_font( "f_yellow.bmp", 32, 96, 10 );
    font->align = ALIGN_X_CENTER | ALIGN_Y_CENTER;
    bricks_load();
    paddle_load();
    ball_load();
    shot_load();
    extras_load();
    frame_create();
    shrapnells_init();
    shine_load();
#ifdef SOUND
    wav_click = sound_chunk_load( "click.wav" );
    wav_damn = sound_chunk_load( "damn.wav" );
    wav_dammit = sound_chunk_load( "dammit.wav" );
    wav_wontgiveup = sound_chunk_load( "wontgiveup.wav" );
    wav_excellent = sound_chunk_load( "excellent.wav" );
    wav_verygood = sound_chunk_load( "verygood.wav" );
#endif
    /* load names of all valid levelsets */
    levelsets_load_names();
}
/*
====================================================================
Delete anything created by game_create();
====================================================================
*/
void game_delete()
{
    free_font( font );
    bricks_delete();
    paddle_delete();
    ball_delete();
    shot_delete();
    extras_delete();
    frame_delete();
    shrapnells_delete();
    shine_delete();
#ifdef SOUND
    if ( wav_click ) sound_chunk_free( wav_click ); wav_click = 0;
    if ( wav_damn ) sound_chunk_free( wav_damn ); wav_damn = 0;
    if ( wav_dammit ) sound_chunk_free( wav_dammit ); wav_dammit = 0;
    if ( wav_wontgiveup ) sound_chunk_free( wav_wontgiveup ); wav_wontgiveup = 0;
    if ( wav_excellent ) sound_chunk_free( wav_excellent ); wav_excellent = 0;
    if ( wav_verygood ) sound_chunk_free( wav_verygood ); wav_verygood = 0;
#endif
    /* free levelset names */
    levelsets_delete_names();
}
/*
====================================================================
Initiates players and the first level.
Return Value: True if successful
====================================================================
*/
int game_init()
{
    int i;
    char path[512];
    char *setname = 0;
    /* load level */
    setname = levelset_names[config.levelset_id];
    if ( levelset_names[config.levelset_id][0] == '~' ) {
        sprintf( path, "%s/%s/levels", getenv( "HOME" ), CONFIG_DIR_NAME );
        setname++;
    }
    else
        sprintf( path, "%s/levels", SRC_DIR );
    if ( !levels_load( path, setname ) ) return 0;
    /* set difficulty */
    diff = diff_get( config.diff );
    /* initiate players from config information */
    players_clear();
    for ( i = 0; i < config.player_count; i++ )
        player_add( config.player_names[i], diff->lives, levels_get_first() );
    player = players_get_first();
    /* initiate level */
    return game_init_level( player, WITH_CREDITS );
}

/*
====================================================================
Free all memory allocated by game_init()
====================================================================
*/
void game_clear()
{
    if ( offscreen ) SDL_FreeSurface( offscreen ); offscreen = 0;
    if ( bkgnd ) SDL_FreeSurface( bkgnd ); bkgnd = 0;
}
/*
====================================================================
Run game after first level was initiated. Initiates next levels,
too.
====================================================================
*/
void game_run()
{
    int leave = 0;
    int next_player = 0;
    int result;
    char str[64];
    int i;

    fade_screen( FADE_IN, FADE_DEF_TIME );
    player_info();
    while ( !leave && !term_game ) {
        /* run game until player wants to quit or restart or lost the ball or reached next level */
        result = breakout_run();
        switch ( result ) {
            case REQUEST_PAUSE:
                confirm( font, "Pause", CONFIRM_PAUSE );
                break;
            case REQUEST_QUIT:
                if ( term_game || confirm( font, "Exit Game? y/n", CONFIRM_YES_NO ) ) 
                    leave = 1;
                break;
            case REQUEST_NEXT_LEVEL:
#ifdef SOUND
                if ( config.speech ) {
                    if ( rand() % 2 )
                        sound_play( wav_excellent );
                    else
                        sound_play( wav_verygood );
                }
#endif
                player->level = levels_get_next( player->level );
                player_reset_bricks( player );
                if ( !player->level ) {
                    confirm( font, "Congratulations! No more levels left!", CONFIRM_ANY_KEY );
                    /* player finished all levels so ''deactivate'' him */
                    player->lives = 0;
                    /* if this was the last player show congrats */
                    if ( players_count() == 0 ) {
                        /* congrats */
                        confirm( font, "Game Over!", CONFIRM_ANY_KEY );
                        leave = 1;
                        break;
                    }
                    else {
                        player = players_get_next();
                        next_player = 1;
                    }
                }
                if ( next_player ) {
                    fade_anims();
                    game_init_level( player, WITH_CREDITS );
                    player_info();
                    next_player = 0;
                }
                else {
                    fade_screen( FADE_OUT, FADE_DEF_TIME );
                    game_init_level( player, WITH_CREDITS );
                    fade_screen( FADE_IN, FADE_DEF_TIME );
                }
                break;
            case REQUEST_RESTART:
                /* if there are no lifes remaining this is a continue request */
                if ( player->lives == 0 ) {
                    sprintf( str, "Buy a continue? (%i points) y/n", diff->life_cost );
                    if ( confirm( font , str, CONFIRM_YES_NO ) ) {
                        player->score -= diff->life_cost;
                        player->lives++;
                    }
                    else
                        break;
                }
                else
                    if ( !confirm( font, "Restart Level? y/n", CONFIRM_YES_NO ) ) break;
                player_reset_bricks( player );
            case REQUEST_NEXT_PLAYER:
                /* if we got here the current player lost a life */
#ifdef SOUND
                if ( result == REQUEST_NEXT_PLAYER && config.speech ) {
                    if ( rand() % 2 )
                        sound_play( wav_damn );
                    else
                        sound_play( wav_dammit );
                }
#endif
                /* if he has no lives left but enough score to buy one he may do so
                else he will be kicked out of the game as lives == 0 */
                if ( player->lives == 0 && player->score >= diff->life_cost ) {
                    sprintf( str, "Buy a continue? (%i points) y/n", diff->life_cost );
                    if ( confirm( font , str, CONFIRM_YES_NO ) ) {
                        player->score -= diff->life_cost;
                        player->lives++;
#ifdef SOUND
                        if ( config.speech )
                            sound_play( wav_wontgiveup );
#endif
                    }
                }
                if ( players_count() >= 1 )
                    player = players_get_next();
                else {
                    /* all players died - quit game */
                    leave = 1;
                    confirm( font, "Game Over!", CONFIRM_ANY_KEY );
                    break;
                }
                fade_anims();
                game_init_level( player, WITH_CREDITS );
                player_info();
                break;
        }
    }
    fade_screen( FADE_OUT, FADE_DEF_TIME );
	/* modify score of involved players */
    if ( diff->score_mod != 1.0 )
		for ( i = 0; i < config.player_count; i++ )
        	players[i].score = (int)(diff->score_mod * players[i].score);
    /* add players to chart */
    chart_clear_new_entry();
    for ( i = 0; i < config.player_count; i++ )
        chart_add( players[i].name, levels_get_id( players[i].level ) + 1, players[i].score );
	/* and save... maybe it crashes and we don't want to loose highscore results, right? */
	chart_save();
}

/*
====================================================================
Test this level until ball gets lost.
====================================================================
*/
void game_test_level( Level *level )
{
    Player test_player;
    /* initate testplayer */
    strcpy( test_player.name, "Player" );
    test_player.score = 0;
    test_player.old_score = 0;
    test_player.lives = 1;
    test_player.level = level;
    player_reset_bricks( &test_player );
    player = &test_player;
    /* difficulty */
    diff = diff_get( config.diff );
    /* init game */
    game_init_level( player, 0 /* no credits */ );
    /* run */
    fade_screen( FADE_IN, FADE_DEF_TIME );
    breakout_run();
    fade_screen( FADE_OUT, FADE_DEF_TIME );
}
