/***************************************************************************
                          manager.c  -  description
                             -------------------
    begin                : Thu Sep 20 2001
    copyright            : (C) 2001 by Michael Speck
    email                : kulkanie@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include "manager.h"
#include "chart.h"
#include "event.h"
#include "config.h"
#ifdef SOUND
#include "audio.h"
#endif
#include "levels.h"

extern Sdl sdl;
extern Config config;
extern List *menus; /* list of menus */
int menu_x = 54, menu_y = 220, menu_w = 256, menu_h = 256, menu_border = 16; /* default region where menu is displayed centered */
int cx = 384, cy = 220, cw = 256, ch = 220; /* center charts here */
Menu *cur_menu; /* current menu */
SDL_Surface *mbkgnd = 0; /* menu background */
Font *mfont = 0, *mhfont = 0, *mcfont = 0; /* font, highlight font, caption font */
int gap_height = 2;
extern char **levelset_names;
extern int levelset_count;
extern char **levelset_home_names;
extern int levelset_home_count;
extern int term_game;
#ifdef SOUND
Sound_Chunk *wav_menu_click = 0;
Sound_Chunk *wav_menu_motion = 0;
#endif
/* some items we need to know to assign name lists later */
Item *item_levelset, *item_set;

/*
====================================================================
Callbacks of menu items.
====================================================================
*/
/* Disable/enable sound */
void cb_sound() {
#ifdef SOUND
    sound_enable( config.sound );
#endif
}
/* set volume */
void cb_volume() {
#ifdef SOUND
    sound_volume( config.volume * 16 );
#endif
}
/* toggle fullscreen */
void cb_fullscreen() {
    manager_show();
}
/* delete set */
void cb_delete_set()
{
    char fname[512];
    /* do not delete <CREATE SET> file */
    if ( strequal( "<CREATE SET>", levelset_home_names[config.levelset_home_id] ) ) {
        printf( "You cannot delete '<CREATE SET>'!\n" );
        return;
    }
    /* get file name + path */
    sprintf( fname, "%s/%s/levels/%s", getenv( "HOME" ), CONFIG_DIR_NAME, levelset_home_names[config.levelset_home_id] );
    remove( fname );
    levelsets_load_names(); /* reinit name lists and configs indices */
    /* reassign these name lists as position in memory has changed */
    value_set_new_names( item_levelset->value, levelset_names, levelset_count );
    value_set_new_names( item_set->value, levelset_home_names, levelset_home_count );
}
/* adjust set list */
void cb_adjust_set_list()
{
    /* reinit name lists and configs indices */
    levelsets_load_names();
    /* reassign these name lists as position in memory has changed */
    value_set_new_names( item_levelset->value, levelset_names, levelset_count );
    value_set_new_names( item_set->value, levelset_home_names, levelset_home_count );
}

/*
====================================================================
Load/delete background and create and link all menus
====================================================================
*/
void manager_create()
{
    Item *key1, *key2, *key3;
    Item *item;
    int filter[SDLK_LAST]; /* key filter */
    /* constant contence of switches */
    char *str_fps[] = { "No Limit", "50 FPS", "100 FPS", "200 FPS" };
    char *str_anim[] = { "Off", "Low", "High" };
    char *str_diff[] = { "Easy", "Medium", "Hard" };
    char *str_ctrl[] = { "Keyboard Only", "Mouse Only", "Keyboard&Mouse" };
    char aux[128];
    /*
    main:
        new game:
            start game
            ---
            levelset
            difficulty
            ---
            active players
            ---
            player1
            player2
            player3
            player4
        controls
        graphics
            animations
            transparency
            shadow
            ---
            display
            constant frame rate
        audio:
            sound
            volume
        advanced options:
            convex paddle
			ball level
        editor:
            set: (list)
            edit
            delete
                yes
                no
            ---
            create empty set
                set name: (string)
                create set
                    yes
                    no
        ---
        quit
    */
    Menu *_main = 0;
#ifdef _1
    Menu *options = 0;
#endif
#ifdef SOUND
    Menu *audio = 0;
#endif
    Menu *gfx = 0;
    Menu *game = 0;
    Menu *cont = 0;
    Menu *adv = 0;
    Menu *editor = 0;
    Menu *confirm_delete = 0;

    /* load graphics and sounds */
    mbkgnd = load_surf( "menuback.bmp", SDL_SWSURFACE );
    SDL_SetColorKey( mbkgnd, 0, 0 );
    mfont = load_fixed_font( "f_small_yellow.bmp", 32, 96, 8 );
    mhfont = load_fixed_font( "f_white.bmp", 32, 96, 10 );
    mcfont = load_fixed_font( "f_yellow.bmp", 32, 96, 10 );
#ifdef SOUND
    wav_menu_click = sound_chunk_load( "click.wav" );
    wav_menu_motion = sound_chunk_load( "motion.wav" );
#endif
    /* add version to background */
    mfont->align = ALIGN_X_RIGHT | ALIGN_Y_BOTTOM;
    sprintf( aux, "v%s", VERSION );
    write_text( mfont, mbkgnd, mbkgnd->w - 2, mbkgnd->h - 2 - mfont->height, aux, OPAQUE );
    write_text( mfont, mbkgnd, mbkgnd->w - 2, mbkgnd->h - 2, "http://www.lgames.org", OPAQUE );

    /* setup filter */
    filter_clear( filter );
    filter_set( filter, SDLK_a, SDLK_z, 1 );
    filter[SDLK_SPACE] = 1;
    filter[SDLK_RSHIFT] = 1;
    filter[SDLK_LSHIFT] = 1;
    filter[SDLK_LALT] = 1;
    filter[SDLK_RALT] = 1;
    filter[SDLK_LCTRL] = 1;
    filter[SDLK_RCTRL] = 1;
    filter[SDLK_UP] = 1;
    filter[SDLK_DOWN] = 1;
    filter[SDLK_LEFT] = 1;
    filter[SDLK_RIGHT] = 1;
    filter[SDLK_q] = 0;
    filter[SDLK_p] = 0;
    filter[SDLK_f] = 0;
    filter[SDLK_s] = 0;
    filter[SDLK_a] = 0;
    filter[SDLK_r] = 0;

    /* menus are added to this list for deleting later */
    menus = list_create( LIST_AUTO_DELETE, menu_delete );
    /* create menus */
    _main   = menu_create( "Menu", MENU_LAYOUT_CENTERED, menu_x, menu_y, menu_w, menu_h, menu_border );
#ifdef _1
    options = menu_create( "Options", MENU_LAYOUT_CENTERED, menu_x, menu_y, menu_w, menu_h, menu_border );
#endif
    gfx     = menu_create( "Graphics", MENU_LAYOUT_CENTERED, menu_x, menu_y, menu_w, menu_h, menu_border );
    game    = menu_create( "New Game", MENU_LAYOUT_CENTERED, menu_x, menu_y, menu_w, menu_h, menu_border );
    cont    = menu_create( "Controls", MENU_LAYOUT_CENTERED, menu_x, menu_y, menu_w, menu_h, menu_border );
    adv     = menu_create( "Advanced Options", MENU_LAYOUT_CENTERED, menu_x, menu_y, menu_w, menu_h, menu_border );
#ifdef SOUND
    audio = menu_create( "Audio", MENU_LAYOUT_CENTERED, menu_x, menu_y, menu_w, menu_h, menu_border );
#endif
    editor  = menu_create( "Editor", MENU_LAYOUT_CENTERED, menu_x, menu_y, menu_w, menu_h, menu_border );
    confirm_delete = menu_create( "Delete Set", MENU_LAYOUT_CENTERED, menu_x, menu_y, menu_w, menu_h, menu_border );
    /* create items */
    /* main menu */
    menu_add( _main, item_create_link    ( "New Game", game ) );
    menu_add( _main, item_create_separator  ( "" ) );
#ifdef _1
    menu_add( _main, item_create_link       ( "Options", options ) );
#endif
    menu_add( _main, item_create_link       ( "Controls", cont ) );
    menu_add( _main, item_create_link       ( "Graphics", gfx ) );
#ifdef SOUND
    menu_add( _main, item_create_link       ( "Audio", audio ) );
#else
    menu_add( _main, item_create_separator  ( "Audio" ) );
#endif
    menu_add( _main, item_create_link       ( "Advanced Options", adv ) );
    menu_add( _main, item_create_separator  ( "" ) );
    menu_add( _main, item_create_link       ( "Editor", editor ) );
    menu_add( _main, item_create_separator  ( "" ) );
    menu_add( _main, item_create_action     ( "Quit", ACTION_QUIT ) );
#ifdef _1
    /* options */
    menu_add( options, item_create_link( "Controls", cont ) );
    menu_add( options, item_create_link( "Graphics", gfx ) );
    menu_add( options, item_create_link( "Audio", audio ) );
    menu_add( options, item_create_separator( "Audio" ) );
    menu_add( options, item_create_separator( "" ) );
    menu_add( options, item_create_link( "Back", _main ) );
#endif
    /* audio */
#ifdef SOUND
    item = item_create_switch( "Sound:", &config.sound, "Off", "On" );
    item->callback = cb_sound;
    menu_add( audio, item );
    menu_add( audio, item_create_switch( "Speech:", &config.speech, "Off", "On" ) );
    item = item_create_range( "Volume:", &config.volume, 1, 8, 1 );
    item->callback = cb_volume;
    menu_add( audio, item );
    menu_add( audio, item_create_separator( "" ) );
    menu_add( audio, item_create_link( "Back", _main ) );
#endif
    /* gfx */
    menu_add( gfx, item_create_switch_x( "Animations:", &config.anim, str_anim, 3 ) );
    menu_add( gfx, item_create_switch( "Transparency:", &config.trp, "Off", "On" ) );
    menu_add( gfx, item_create_switch( "Shadow:", &config.shadow, "Off", "On" ) );
    menu_add( gfx, item_create_separator( "" ) );
    item = item_create_switch( "Display:", &config.fullscreen, "Window", "Fullscreen" );
    item->callback = cb_fullscreen;
    menu_add( gfx, item );
    menu_add( gfx, item_create_switch_x( "Frame Rate:", &config.fps, str_fps, 4 ) );
    menu_add( gfx, item_create_separator( "" ) );
    menu_add( gfx, item_create_link( "Back", _main ) );
    /* game */
    menu_add( game, item_create_action( "Start Game", ACTION_PLAY ) );
    menu_add( game, item_create_separator( "" ) );
    item_levelset = item_create_switch_x( "Levelset:", &config.levelset_id, levelset_names, levelset_count );
    menu_add( game, item_levelset );
    menu_add( game, item_create_switch_x( "Difficulty:", &config.diff, str_diff, 3 ) );
    menu_add( game, item_create_separator( "" ) );
    menu_add( game, item_create_range( "Players:", &config.player_count, 1, 4, 1 ) );
    menu_add( game, item_create_edit( "1st:", config.player_names[0], 12 ) );
    menu_add( game, item_create_edit( "2nd:", config.player_names[1], 12 ) );
    menu_add( game, item_create_edit( "3rd:", config.player_names[2], 12 ) );
    menu_add( game, item_create_edit( "4th:", config.player_names[3], 12 ) );
    menu_add( game, item_create_separator( "" ) );
    menu_add( game, item_create_link( "Back", _main ) );
    /* controls */
    key1 = item_create_key( "Left:", &config.k_left, filter );
    key2 = item_create_key( "Right:", &config.k_right, filter );
    key3 = item_create_key( "Fire:", &config.k_fire, filter );
    menu_add( cont, key1 );
    menu_add( cont, key2 );
    menu_add( cont, key3 );
    /* dynamic restriction */
    value_add_other_key( key1->value, key2->value );
    value_add_other_key( key1->value, key3->value );
    value_add_other_key( key2->value, key1->value );
    value_add_other_key( key2->value, key3->value );
    value_add_other_key( key3->value, key1->value );
    value_add_other_key( key3->value, key2->value );
    /* other control stuff */
    menu_add( cont, item_create_separator( "" ) );
    menu_add( cont, item_create_switch( "Grab Mouse:", &config.grab, "Off", "On" ) );
    menu_add( cont, item_create_switch( "Relative Motion:", &config.rel_motion, "Off", "On" ) );
    menu_add( cont, item_create_range( "Motion Modifier:", &config.motion_mod, 40, 160, 10 ) );
    menu_add( cont, item_create_switch( "Invert Motion:", &config.invert, "Off", "On" ) );
    menu_add( cont, item_create_separator( "" ) );
    menu_add( cont, item_create_switch_x( "Use:", &config.control, str_ctrl, 3 ) );
    menu_add( cont, item_create_separator( "" ) );
    menu_add( cont, item_create_link( "Back", _main ) );
    /* advanced options */
    menu_add( adv, item_create_switch( "Convex Paddle:", &config.convex, "Off", "On" ) );
    menu_add( adv, item_create_switch( "Balls:", &config.ball_level, "Below Bonuses", "Above Bonuses" ) );
    menu_add( adv, item_create_separator( "" ) );
    menu_add( adv, item_create_link( "Back", _main ) );
    /* editor */
    item_set = item_create_switch_x( "Set:", &config.levelset_home_id, levelset_home_names, levelset_home_count );
    menu_add( editor, item_set );
    menu_add( editor, item_create_action( "Edit Set", ACTION_EDIT ) );
    menu_add( editor, item_create_link( "Delete Set", confirm_delete ) );
    menu_add( editor, item_create_separator( "" ) );
    menu_add( editor, item_create_link( "Back", _main ) );
    /* confirm_delete set */
    item = item_create_link( "Yes", editor );
    item->callback = cb_delete_set;
    menu_add( confirm_delete, item );
    menu_add( confirm_delete, item_create_link( "No", editor ) );

    /* adjust all menus */
    menu_adjust( _main );
#ifdef _1
    menu_adjust( options );
#endif
#ifdef SOUND
    menu_adjust( audio );
#endif
    menu_adjust( gfx );
    menu_adjust( game );
    menu_adjust( cont );
    menu_adjust( adv );
    menu_adjust( editor );
    menu_adjust( confirm_delete );
    /* set main menu as current */
    menu_select( _main );
}
void manager_delete()
{
    list_delete( menus );
    if ( mbkgnd ) SDL_FreeSurface( mbkgnd ); mbkgnd = 0;
    if ( mfont ) free_font( mfont ); mfont = 0;
    if ( mhfont ) free_font( mhfont ); mhfont = 0;
    if ( mcfont ) free_font( mcfont ); mcfont = 0;
#ifdef SOUND
    if ( wav_menu_click ) sound_chunk_free( wav_menu_click ); wav_menu_click = 0;
    if ( wav_menu_motion ) sound_chunk_free( wav_menu_motion ); wav_menu_motion = 0;
#endif
}
/*
====================================================================
Run menu until request sent
====================================================================
*/
int manager_run()
{
    SDL_Event event;
    int event_polled = 0; /* event occured? */
    int result = ACTION_NONE;
    int ms;
    /* draw highscores */
    chart_show( cx, cy, cw, ch );
    /* loop */
    reset_timer();
    while ( result == ACTION_NONE && !term_game ) {
        if ( event_poll( &event ) ) event_polled = 1;
        if ( event_polled && event.type == SDL_QUIT ) {
            result = ACTION_QUIT;
            term_game = 1;
        }
		/* fullscreen if no item selected */
		if ( event_polled )
			if ( cur_menu->cur_item == 0 || ( cur_menu->cur_item->type != ITEM_EDIT && cur_menu->cur_item->type != ITEM_KEY ) )
				if ( event.type == SDL_KEYUP )
					if ( event.key.keysym.sym == SDLK_f ) {
						config.fullscreen = !config.fullscreen;
						set_video_mode( std_video_mode( config.fullscreen ) );
						FULL_DEST( sdl.screen ); FULL_SOURCE( mbkgnd ); blit_surf();
					    chart_show( cx, cy, cw, ch );
						refresh_screen( 0, 0 ,0, 0 );
					}
        ms = get_time();
        menu_hide( cur_menu );
        if ( event_polled )
            result = menu_update( cur_menu, &event, ms );
        else
            result = menu_update( cur_menu, 0, ms );
        menu_show( cur_menu );
        refresh_rects();
        event_polled = 0;
        SDL_Delay( 5 );
    }
    return result;
}
/*
====================================================================
Fade in/out background of menu
====================================================================
*/
void manager_fade( int type )
{
    if ( type == FADE_IN ) {
        FULL_DEST( sdl.screen ); FULL_SOURCE( mbkgnd ); blit_surf();
    }
    fade_screen( type, FADE_DEF_TIME );
}
/*
====================================================================
Update screen without menu itself as this is shown next frame.
====================================================================
*/
void manager_show()
{
    set_video_mode( std_video_mode( config.fullscreen ) );
    FULL_DEST( sdl.screen ); FULL_SOURCE( mbkgnd ); blit_surf();
    chart_show( cx, cy, cw, ch );
    add_refresh_rect( 0, 0, sdl.screen->w, sdl.screen->h );
}

/*
====================================================================
Update set list when creating a new file for editor.
====================================================================
*/
void manager_update_set_list()
{
    cb_adjust_set_list(); /* hacky but shiiiit how cares? */
}
