/***************************************************************************
                          shrapnells.c  -  description
                             -------------------
    begin                : Sat Sep 8 2001
    copyright            : (C) 2001 by Michael Speck
    email                : kulkanie@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lbreakout.h"
#include "config.h"
#include "list.h"
#include "shrapnells.h"

extern Config config;
extern Sdl sdl;
extern SDL_Surface *offscreen;
List *shrapnells = 0;

/*
====================================================================
Local
====================================================================
*/

/*
====================================================================
Create a single shrapnell and add to list.
====================================================================
*/
void shrapnell_create( int x, int y, int w, int h, float vx, float vy )
{
    Shrapnell *shr = calloc( 1, sizeof( Shrapnell ) );
    shr->pic = create_surf(w, h, SDL_HWSURFACE);
    SDL_SetColorKey(shr->pic, 0, 0);
    DEST(shr->pic, 0, 0, w, h);
    SOURCE(offscreen, x, y);
    blit_surf();
    shr->x = x;
    shr->y = y;
    shr->v.x = vx;
    shr->v.y = vy;
    shr->alpha = 0;
    list_add( shrapnells, shr);
}
/*
====================================================================
Delete a shrapnell
====================================================================
*/
void shrapnell_delete( void *ptr )
{
    Shrapnell *shr = (Shrapnell*)ptr;
    SDL_FreeSurface( shr->pic );
    free( shr );
}

/*
====================================================================
Public
====================================================================
*/

/*
====================================================================
Initate or delete shrapnell list
====================================================================
*/
void shrapnells_init()
{
    shrapnells = list_create( LIST_AUTO_DELETE, shrapnell_delete );
}
void shrapnells_delete()
{
    list_delete( shrapnells );
}
/*
====================================================================
Create shrapnells from position in offscreen
====================================================================
*/
void shrapnells_create( int x, int y, int w, int h, int res )
{
    int mod = config.anim;
    int i, j;
    int r;
    int shr_w, shr_h, dx, dy;

    /* no animations? */
    if (!config.anim) return;

    /* high/low resolution shrapnells? */
    if ( res == SHR_LOW_RES )
        r = rand() % 3;
    else {
        r = rand() % 5;
        if (r == 4) r = 3;
    }

    /* check result */
    switch (r) {
        case 0:
            shr_w = w / (8 * mod);
            shr_h = h;
            for (i = 0; i < (4 * mod); i++)
                shrapnell_create(x + i * shr_w, y, shr_w, shr_h, -0.05 - i * 0.01, 0);
            for (i = (4 * mod) - 1; i >= 0; i--)
                shrapnell_create(x + w - (i + 1) * shr_w, y, shr_w, shr_h, 0.05 + i * 0.01, 0);
            break;
        case 1:
            shr_w = w;
            shr_h = h / (4 * mod);
            for (i = 0; i < (2 * mod); i++)
                shrapnell_create(x, y + i * shr_h, shr_w, shr_h, 0, -0.05 - i * 0.01);
            for (i = (2 * mod) - 1; i >= 0; i--)
                shrapnell_create(x, y + h - (i + 1) * shr_h, shr_w, shr_h, 0, 0.05 + i * 0.01);
            break;
        case 2:
            shr_w = w / (8 * mod);
            shr_h = h;
            for (i = 0; i < (8 * mod); i++)
                shrapnell_create(x + i * shr_w, y, shr_w, shr_h, 0, (1 - 2 * (i & 1)) * 0.1);
            break;
        case 3:
            shr_w = config.anim == 1 ? 8 : 4;
            shr_h = config.anim == 1 ? 8 : 4;
            for (i = 0; i < w / shr_w; i++)
                for (j = 0; j < h / shr_h; j++) {
                    dx = rand() % 2 == 0 ? 1 : -1;
                    dy = rand() % 2 == 0 ? 1 : -1;
                    shrapnell_create(x + i * shr_w, y + j * shr_h, shr_w, shr_h, (float)(rand()%6+5) / 100 * dx, (float)(rand()%6+5) / 100 * dy);
                 }
            break;
    }
}
/*
====================================================================
Remove all shrapnells
====================================================================
*/
void shrapnells_reset()
{
    list_clear( shrapnells );
}
/*
====================================================================
Show, hide all shrapnells.
====================================================================
*/
void shrapnells_hide()
{
    List_Entry  *entry = shrapnells->head.next;
    Shrapnell   *shr;
    int         x, y;
    while ( entry != &shrapnells->tail ) {
        shr = entry->item;
        x = (int)shr->x;
        y = (int)shr->y;
        DEST(sdl.screen, x, y, shr->pic->w, shr->pic->h);
        SOURCE(offscreen, x, y);
        blit_surf();
        add_refresh_rect(x, y, shr->pic->w, shr->pic->h);
        entry = entry->next;
    }
}
void shrapnells_show()
{
    List_Entry  *entry = shrapnells->head.next;
    Shrapnell   *shr;
    int         x, y;
    while ( entry != &shrapnells->tail ) {
        shr = entry->item;
        x = (int)shr->x;
        y = (int)shr->y;
        DEST(sdl.screen, x, y, shr->pic->w, shr->pic->h);
        SOURCE(shr->pic, 0, 0);
        if (!config.trp || shr->alpha == 0)
            blit_surf();
        else
            alpha_blit_surf((int)shr->alpha);
        add_refresh_rect(x, y, shr->pic->w, shr->pic->h);
        entry = entry->next;
    }
}
/*
====================================================================
Update and remove expired shrapnells
====================================================================
*/
void shrapnells_update( int ms )
{
    List_Entry  *entry = shrapnells->head.next;
    Shrapnell   *shr;
    while ( entry != &shrapnells->tail ) {
        shr = entry->item;
        shr->x += shr->v.x * ms;
        shr->y += shr->v.y * ms;
        if (shr->alpha < 255)
            shr->alpha += 0.20 * ms;
        entry = entry->next;
        if (shr->alpha > 255 || shr->x + shr->pic->w < 0 || shr->y + shr->pic->h < 0 || shr->x > sdl.screen->w || shr->y > sdl.screen->h)
            list_delete_entry( shrapnells, entry->prev );
    }
}
